'use strict';

/**
 * Exports Yargs commands
 * @see https://github.com/yargs/yargs/blob/main/docs/advanced.md
 * @private
 * @module
 */

module.exports = {
  init: require('./init'),
  // default command
  run: require('./run'),
}
