html {
  font: 16px/1.6 'Helvetica Neue', Helvetica, Arial, sans-serif;
}

body {
  border-top: 2px solid #ddd;
  color: #2c2c2c;
}

@keyframes fadein {
  from {
    opacity: 0;
  }

  to {
    opacity: 1;
  }
}

@keyframes slideright {
  from {
    transform: translateX(-45px);
  }

  to {
    transform: translateX(0);
  }
}

header,
#content {
  margin: 0 auto;
  max-width: 920px;
  padding-left: 30px;
  padding-right: 30px;
}

header {
  padding-top: 20px;
}

#mocha-logo {
  height: 192px;
  width: 192px;
}

#content {
  padding-bottom: 30px;
}

#_header h1 {
  animation: fadein 1s forwards;
  margin-top: 0;
  margin-left: -19px;
  opacity: 0;
}

#tag {
  animation: fadein 1s forwards, slideright 1s forwards;
  color: #c29d7f;
  font-size: 30px;
  font-weight: 100;
  letter-spacing: 2px;
  margin-top: -158px;
  margin-left: 185px;
  margin-bottom: 125px;
}

#tag em {
  font-style: normal;
}

nav.badges a + a {
  margin-left: 3px;
}

ul.image-list {
  overflow: hidden;
  text-align: center;
  list-style: none;
  column-count: 1;
  padding: 0;
  margin: 0;
}

ul.image-list li {
  border-bottom: none;
  display: inline-block;
  margin: 0 4px 0 4px;
  max-height: 64px;
  padding: 0;
  line-height: 0;
}

ul.image-list li a {
  display: inline-block;
}

ul.image-list li img {
  margin: 0;
  padding: 0;
  display: block;
  max-height: 64px;
}

ul#backers.image-list li img {
  width: 32px;
  height: 32px;
  overflow: hidden;
}

.faded-image {
  background-color: rgba(0, 0, 0, 0.05);
}

.faded-image.is-loaded {
  background-color: rgba(0, 0, 0, 0);
  transition: background-color 0.3s;
}

.faded-image img {
  opacity: 0;
  transition: opacity 0.1s;
}

.faded-image.is-loaded img {
  opacity: 1;
}

h2 {
  border-bottom: 1px solid #ddd;
  font-weight: 400;
  letter-spacing: 1px;
  margin-top: 80px;
  text-transform: uppercase;
}

h3 {
  border-bottom: 1px solid #eee;
  font-weight: 200;
  letter-spacing: 1px;
  margin-top: 40px;
  text-transform: uppercase;
}

h3 > code {
  font-size: 14px;
  text-transform: none;
}

#content > p:first-child {
  font-size: 20px;
  font-weight: 200;
  letter-spacing: 1px;
}

a {
  color: #8d6748;
}

a:hover {
  color: #717171;
}

a.direct-link {
  background: url(../images/link-icon.svg) center center no-repeat;
  background-size: auto 60%;
  opacity: 0;
  overflow: hidden;
  position: absolute;
  text-decoration: none;
  text-indent: -60px;
  transform: translateX(-100%);
  width: 30px;
}

:hover > a.direct-link {
  opacity: 1;
}

ul {
  box-sizing: content-box;
}

ul.two-column {
  column-count: 2;
  column-gap: 30px;
  margin-top: 20px;
  padding: 0 15px;
}

ul.single-column,
ul.single-column > li > ul {
  margin-top: 0;
  padding: 0 0 0 15px;
}

ul li {
  border-bottom: 1px solid #eee;
  break-inside: avoid;
  list-style: none;
  margin-top: 5px;
  padding: 5px 0;
}

ul.single-column li,
ul.single-column li > ul > li {
  list-style: disc;
  padding: 0;
  border-bottom: 0;
}

ul.single-column li > ul > li {
  list-style: circle;
}

code {
  font: 14px monaco, monospace;
  line-height: 1.8;
}

:not(pre) > code {
  background-color: #f5f2f0;
  border-radius: 3px;
  padding: 0.2em 0.4em;
}

pre {
  background-color: #f3f3f3;
  border: 1px solid #ddd;
  border-bottom-color: #ccc;
  border-radius: 3px;
  box-shadow: inset 0 0 10px #ddd;
  margin: 20px 0;
  overflow-x: auto;
  padding: 20px;
}

img.screenshot {
  border-radius: 3px;
  box-shadow: 0 3px 10px #dedede, 0 1px 5px #888;
  display: block;
  margin: 30px auto;
  max-width: 100%;
}

footer {
  background-color: #eee;
  border-top: 1px solid #ddd;
  padding: 50px 30px;
  color: #888;
  font-size: 0.8em;
}

#last-modified {
  text-align: right;
  font-style: italic;
  font-size: 0.7em;
}

#copyright-notice {
  max-width: 66%;
}

#external-links {
  position: relative;
}

#openjsf-links {
  margin: 0;
  padding: 0;
  box-sizing: initial;
  columns: initial;
}

#openjsf-links li {
  display: inline; /* displays list horizontally */
  border: none;
  list-style: none;
  margin: initial;
  padding: initial;
}

/* displays each list item with a "|" between */
#openjsf-links li + li:before {
  content: ' | ';
}

.openjsf-logo {
  margin-bottom: 10px;
}

#external-links .netlify-badge {
  position: absolute;
  right: 0;
  bottom: 0;
}

.sponsorship {
  display: flex;
  align-items: center;
  justify-content: center;
  margin-bottom: 30px;
}

.sponsorship a {
  display: inline-block;
  margin-right: 15px;
  margin-left: 15px;
}

.dl-inline dt,
.dl-inline dd {
  display: inline;
  margin: 0;
}

.dl-inline dt:after {
  content: ': ';
}

.dl-inline dd + dt:before {
  content: '';
  display: block;
}

blockquote {
  border-left: 1px solid #eee;
  padding: 10px;
}

table {
  width: 100%;
}

table td {
  border-top: 1px solid #eee;
  vertical-align: top;
  word-break: break-all;
}

@media all and (max-width: 960px) {
  #copyright-notice {
    max-width: initial;
  }

  #openjsf-links {
    width: 50%;
    column-count: 2;
    column-rule: 1px solid #dedede;
  }

  #openjsf-links li + li:before {
    content: initial;
  }
}

@media all and (max-width: 600px) {
  #tag {
    margin-top: 0;
    margin-left: 0;
    margin-bottom: 0;
  }

  ul {
    column-count: 1;
  }

  pre {
    margin: 20px -11px;
    padding: 10px;
  }

  /* this list is too long for smaller displays */
  #openjsf-links {
    width: 100%;
    columns: initial;
  }

  #openjsf-links li {
    display: block;
  }
}

#mocha-fixture-wizard text[font-weight=bold]:hover {
  text-decoration: underline;
}
