.image-list-wide a,
.image-list-wide li {
  width: 100%;
}

.backer,
.sponsor {
  background-position: center;
  background-repeat: no-repeat;
  background-size: contain;
}

.sponsor {
  height: 64px;
  width: 100%;
}

.backer {
  width: 32px;
  height: 32px;
}
